/*
   Copyright (c) 2009-2016, Jack Poulson
   All rights reserved.

   This file is part of Elemental and is under the BSD 2-Clause License,
   which can be found in the LICENSE file in the root directory, or at
   http://opensource.org/licenses/BSD-2-Clause
*/
#ifndef EL_CONFIG_H
#define EL_CONFIG_H

#include "hydrogen_config.h"

/* Build type and version information */
#define EL_GIT_SHA1 ""
#define EL_VERSION_MAJOR "1"
#define EL_VERSION_MINOR "3"
#define EL_VERSION_PATCH "3"
#define EL_CMAKE_BUILD_TYPE "Release"

#ifdef HYDROGEN_RELEASE_BUILD
#define EL_RELEASE
#endif

#ifdef Hydrogen_EXPORTS
#define EL_EXPORTS
#endif

/* #undef EL_HYBRID */
#define BUILD_SHARED_LIBS
/* #undef MSVC */

/* Compiler information */
#define EL_CMAKE_C_COMPILER   "/opt/bin/arm-linux-musleabihf-gcc"
#define EL_CMAKE_CXX_COMPILER "/opt/bin/arm-linux-musleabihf-g++"
#define EL_CXX_FLAGS          ""
/* #undef EL_HAVE_F90_INTERFACE */
#ifdef EL_HAVE_F90_INTERFACE
# include "El/FCMangle.h"
#endif
#define EL_FORT_LOGICAL 
#define EL_FORT_TRUE    
#define EL_FORT_FALSE   

/* Message Passing Interface */
#define EL_MPI_C_COMPILER        ""
#define EL_MPI_C_INCLUDE_PATH    ""
#define EL_MPI_C_COMPILE_FLAGS   ""
#define EL_MPI_C_LIBRARIES       ""
#define EL_MPI_C_LINK_FLAGS      ""
#define EL_MPI_CXX_COMPILER      "/workspace/destdir/bin/mpicxx"
#define EL_MPI_CXX_INCLUDE_PATH  "/workspace/destdir/include"
#define EL_MPI_CXX_COMPILE_FLAGS ""
#define EL_MPI_CXX_LIBRARIES     "/opt/arm-linux-musleabihf/arm-linux-musleabihf/sys-root/usr/local/lib/libmpicxx.so;/opt/arm-linux-musleabihf/arm-linux-musleabihf/sys-root/usr/local/lib/libmpi.so"
#define EL_MPI_CXX_LINK_FLAGS    "-Wl,-rpath -Wl,/workspace/destdir/lib -Wl,--enable-new-dtags -L/workspace/destdir/lib"

/* Math libraries */
#define EL_MATH_LIBS ""

/* #undef EL_BUILT_BLIS_LAPACK */
#define EL_HAVE_OPENBLAS
/* #undef EL_BUILT_OPENBLAS */

#define HYDROGEN_BLAS_SUFFIX _
#define HYDROGEN_LAPACK_SUFFIX _

/* #undef EL_HAVE_SCALAPACK */
/* #undef EL_BUILT_SCALAPACK */
/* #undef EL_HAVE_SCALAPACK_SUFFIX */
#ifdef EL_HAVE_SCALAPACK_SUFFIX
 #define EL_SCALAPACK_SUFFIX 
#endif

/* #undef EL_HAVE_FLA_BSVD */
/* #undef EL_HAVE_QUAD */
/* #undef EL_HAVE_QUADMATH */
/* #undef EL_HAVE_QD */
/* #undef EL_HAVE_MPC */
/* #undef EL_HAVE_MKL */
/* #undef EL_HAVE_MKL_GEMMT */
/* #undef EL_DISABLE_MKL_CSRMV */

/* Miscellaneous configuration options */
#define EL_RESTRICT __restrict__
#define EL_HAVE_PRETTY_FUNCTION
/* #undef EL_HAVE_OPENMP */
/* #undef EL_HAVE_OMP_COLLAPSE */
/* #undef EL_HAVE_OMP_SIMD */
/* #undef EL_HAVE_QT5 */
/* #undef EL_AVOID_COMPLEX_MPI */
/* #undef EL_USE_BYTE_ALLGATHERS */
/* #undef EL_USE_64BIT_INTS */
/* #undef EL_USE_64BIT_BLAS_INTS */

/* Sparse-direct configuration */
/* #undef EL_USE_CUSTOM_ALLTOALLV */

/* Advanced configuration options */
/* #undef EL_ZERO_INIT */
/* #undef EL_CACHE_WARNINGS */
/* #undef EL_UNALIGNED_WARNINGS */
/* #undef EL_VECTOR_WARNINGS */

#ifdef BUILD_SHARED_LIBS
# if defined _WIN32 || defined __CYGWIN__
#  ifdef __GNUC__ /* Compiling with GNU on Windows */
#   ifdef EL_EXPORTS
#    define EL_EXPORT __attribute__ ((dllexport))
#   else
#    define EL_EXPORT __attribute__ ((dllimport))
#   endif
#  else  /* Compiling with non-GNU on Windows (check for MSVC?) */
#   ifdef EL_EXPORTS
#    define EL_EXPORT __declspec(dllexport)
#   else
#    define EL_EXPORT __declspec(dllimport)
#   endif
#  endif
#  define EL_LOCAL
# else
#  if __GNUC__ >= 4
#   define EL_EXPORT __attribute__ ((visibility ("default")))
#   define EL_LOCAL  __attribute__ ((visibility ("hidden")))
#  else
#   define EL_EXPORT
#   define EL_LOCAL
#  endif
# endif
#else
# define EL_EXPORT
# define EL_LOCAL
#endif

/* #undef EL_HAVE_VALGRIND */

#endif /* EL_CONFIG_H */
